#!/usr/bin/env python3

import logging
import click
import click_log
import json
import re
from time import gmtime, strftime
from dotenv import load_dotenv
from influxdb import InfluxDBClient
from pyzabbix import ZabbixMetric, ZabbixSender # Only works on python2

from .endpoint import SummerEndpoint

load_dotenv()

logger = logging.getLogger(name='summmo')
click_log.basic_config(logger)

@click.group()
@click.option('--password', help="SUMMER ssh management password", envvar="SUMMMO_PASSWORD")
@click.option('--endpoint', help="SUMMER ssh management host", envvar="SUMMMO_ENDPOINT")
@click.option('--endpoint-name', help="SUMMER ssh management host name. Default is the shortname of endpoint", envvar="SUMMMO_ENDPOINT_NAME")
@click.option('--login', default="vsadmin", help="SUMMER ssh management login", envvar="SUMMMO_LOGIN")
@click_log.simple_verbosity_option(logger)
@click.pass_context
def cli(ctx, endpoint, endpoint_name, password, login):
    logger.debug(f"endpoint: {endpoint}, prefix: {endpoint_name}")
    ctx.ensure_object(dict)
    ctx.obj['password'] = password
    ctx.obj['endpoint'] = endpoint
    ctx.obj['login'] = login
    if not endpoint_name:
        endpoint_name = endpoint.split('.')[0]
    ctx.obj['endpoint_name'] = endpoint_name


@cli.command(help="Make Zabbix LLD data")
@click.pass_context
def lld(ctx):
    """
    Command line interface
    """
    ep = SummerEndpoint(ctx.obj['endpoint_name'],
                        ctx.obj['endpoint'],
                        username=ctx.obj['login'],
                        password=ctx.obj['password'])
    lld = ep.zabbix_lld()
    print(json.dumps(lld))

@cli.command(help="Fetch all data for all volumes and report")
@click.pass_context
@click.option('--zabbix-server', help="Report to zabbix server", envvar="SUMMMO_REPORT_ZABBIX_SERVER")
@click.option('--zabbix-agent', help="Zabbix agent name", envvar="SUMMMO_REPORT_ZABBIX_AGENT")
@click.option('--influxdb', help="Report to influxdb", is_flag=True, default=False, envvar="SUMMMO_REPORT_INFLUXDB")
@click.option('--influxdb-measurement', help="Influxdb measurement", envvar="SUMMMO_REPORT_INFLUXDB_MEASUREMENT")
@click.option('--influxdb-host', help="Influxdb host", envvar="SUMMMO_REPORT_INFLUXDB_HOST")
@click.option('--influxdb-port', help="Influxdb port", envvar="SUMMMO_REPORT_INFLUXDB_PORT", default=8086)
@click.option('--influxdb-database', help="Influxdb database", envvar="SUMMMO_REPORT_INFLUXDB_DATABASE")
@click.option('--influxdb-user', help="Influxdb user", envvar="SUMMMO_REPORT_INFLUXDB_USER")
@click.option('--influxdb-password', help="Influxdb password", envvar="SUMMMO_REPORT_INFLUXDB_PASSWORD")
@click.option('--influxdb-rp', help="Influxdb retention policy", envvar="SUMMMO_REPORT_INFLUXDB_RP")
def report(ctx, zabbix_server, zabbix_agent, influxdb,
           influxdb_measurement, influxdb_host, influxdb_port, influxdb_database, influxdb_user, influxdb_password, influxdb_rp):
    ep = SummerEndpoint(ctx.obj['endpoint_name'],
                        ctx.obj['endpoint'],
                        username=ctx.obj['login'],
                        password=ctx.obj['password'])
    ep.fetch_data()
    if influxdb:
        influxdb_json_data = []
        for _, vol in ep.volumes.items():
            influxdb_json_data.append(
                {"measurement": influxdb_measurement,
                    "tags": {
                        "lif": ctx.obj['endpoint_name'],
                        "volume": vol.name,
                        "qtree": None
                    },
                    "time": strftime("%Y-%m-%dT%H:%M:%SZ", gmtime()),
                    "fields": {
                        "available": vol.data['available_size'],
                        "used": vol.data['used_size'],
                        "total": vol.data['total_size'],
                        "snapshot_used": vol.data['snapshot_size'],
                        "snapshot_reserve": vol.data['snapshot_reserve']
                    }
                }
            )
        try:
            logger.info("Sending data to influxdb")
            logger.debug("host     = %s", influxdb_host)
            logger.debug("port     = %s", influxdb_port)
            logger.debug("database = %s", influxdb_database)
            logger.debug("username = %s", influxdb_user)
            client = InfluxDBClient(host     = influxdb_host,
                                    port     = influxdb_port,
                                    database = influxdb_database,
                                    username = influxdb_user,
                                    password = influxdb_password,
                                    ssl      = True,
                                    verify_ssl = False)
            client.write_points(influxdb_json_data, retention_policy=influxdb_rp)
        except Exception as err:
            logger.error("Unexpected error writing data to influxdb")
            logger.error(err)
    if zabbix_server:
        zabbix_packet = []
        for _, vol in ep.volumes.items():
            for key, value in vol.data.items():
                zabbix_packet.append(ZabbixMetric(zabbix_agent,
                                              f"summer.{ key }[{ ctx.obj['endpoint_name'] },{ vol.name }]",
                                              vol.data["snapshot_size"]))
        try:
            ZabbixSender(zabbix_server=zabbix_server).send(zabbix_packet)
        except Exception as err:
            logging.error("Unexpected error writing data to Zabbix")
            logging.error(err)

    logger.info("Done")


if __name__ == '__main__':
    cli(ctx={}, auto_envvar_prefix='SUMMMO')
